
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include "behavior.h"

#define RESIZE_FACTOR (2)

char *behavior_library_path = NULL;
GList *behavior_libraries = NULL;

GHashTable *behavior_func_names = NULL;

/* array of pointers to functions that take a
 * Map* and a Mob* as their arguments 
 */
int (**behavior_funcs)(Map* map,Mob *self) = NULL;
int behavior_funcs_size = 0;
int behavior_funcs_used = 0;


void add_behavior_library(char *filename) {

  char *str = g_module_build_path(behavior_library_path,filename);
  GModule *mod = g_module_open(str,G_MODULE_BIND_LAZY);

  if(mod != NULL) behavior_libraries = g_list_append(behavior_libraries,mod);
  if(str != NULL) free(str);

}

void *add_behavior(char *name) {

  GList *node = behavior_libraries;
  int (*funcptr)(Map *map,Mob *self);

  while(node != NULL) { /* check all libraries for the symbol name */
    if(node->data != NULL)
      if(g_module_symbol(((GModule*)node->data),name,(gpointer)&funcptr)) 
	return funcptr; /* found it... */
    node = node->next; /* didn't find it, move on. */
  }
  return NULL;
}

void resize_behavior_funcs(int size) {

  int (**nb)(Map*,Mob*) = NULL;
  
  /* whoa, look at that typecast *grin* an array of pointers to functions
   * that take a pointer to a Map and a pointer to a Mob and return an int.
   */
  if((nb=(int(**)(Map*,Mob*))realloc(behavior_funcs,
				     sizeof(int*)*size))==NULL) {
    fprintf(stderr,"%s: couldn't allocate new behavior_funcs array\n",
	    __FUNCTION__);
    exit(1);
  } else behavior_funcs = nb;

  behavior_funcs_size = size;
  memset(behavior_funcs + behavior_funcs_used, 0,
	 (behavior_funcs_size - behavior_funcs_used) * sizeof(int*)); 
  
}

int load_behavior(char *name) {

  gpointer n;

  if(behavior_func_names == NULL)
    behavior_func_names = g_hash_table_new(g_str_hash,g_str_equal);
  else if((n = g_hash_table_lookup(behavior_func_names,name)) != NULL) 
    return GPOINTER_TO_INT(n-1); /* values are 1, 2, ..., NULL = error */

  if(behavior_funcs_size == behavior_funcs_used) {
    if(behavior_funcs_size == 0) behavior_funcs_size = 1;
    resize_behavior_funcs(behavior_funcs_size *= RESIZE_FACTOR);
  }

  behavior_funcs[behavior_funcs_used] = add_behavior(name);
  if(behavior_funcs[behavior_funcs_used] == NULL) {
    fprintf(stderr,"%s: Couldn't load behavior %s\n",__FUNCTION__,name);
    return -1;
  }

  /* +1 because g_hash_table_insert() returns 0 on failure,
   * while that could be a valid array index in behavior_funcs[]
   */
  g_hash_table_insert(behavior_func_names,name,
		      GINT_TO_POINTER(behavior_funcs_used+1));

  return behavior_funcs_used++;

}
