
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include "draw.h"

SDL_Surface *screen;      /* The actual video buffer */
int screen_w = 640;
int screen_h = 480;

SDL_Rect *update = NULL;  /* an array of SDL_Rect's with the areas that
			     have to be updated in SDL_Surface screen. */
int update_size = 0;
Uint32 video_flags = 0;

GList *update_coords = NULL; /* map coordinates that have to be updated */

int viewport[4] = {0,0,0,0}; /* the coords of the visible part of the map */
int tile_size = 12;          /* a very important number *grin* */

void draw_tile(SDL_Surface *image,int x,int y,int xo,int yo) {
 
  SDL_Rect dest_rect = {x,y,tile_size,tile_size},
            src_rect = {xo,yo,tile_size,tile_size};

  if(SDL_BlitSurface(image, &src_rect, screen, &dest_rect) < 0)
    fprintf(stderr,"%s: BlitSurface error: %s\n",__FUNCTION__,SDL_GetError());

  /* add our rectangle to the update rectangle */
  add_update(x,y,tile_size,tile_size);

}

void draw_map(Map *map) {


 /* clear the update rectangles, it's possible that other functions
  * have used 'update' in the meantime...
  */
  clear_update();

  while(update_coords != NULL) { /* there's coordinates to draw */
    int x, y;
    if(update_coords->data == NULL)
        fprintf(stderr,"%s: error while parsing update_coords\n",__FUNCTION__);

    x = ((int*)update_coords->data)[0];
    y = ((int*)update_coords->data)[1];

    /* DEBUG STUFF... */
    //fprintf(stdout,"%s: drawing (%d,%d)\n",__FUNCTION__,x,y);

    if(inside_rect(viewport[0],viewport[1],viewport[2],viewport[3],x,y)) {
      int n, *img = NULL,
	x_offset = ((x - viewport[0]) * tile_size), /* screen pixel offsets */
	y_offset = ((y - viewport[1]) * tile_size);
      Mob *mob = NULL;
      Space *space = (Space*)map_get_space(map,x,y); /* the space to draw */

      img = (int*)space->data[SPACE_DATA_BOTTOM_IMAGE];
      if(img[0] != -1) {
	//fprintf(stdout,"%s: bottom\n",__FUNCTION__);
	draw_tile(images[img[0]],x_offset,y_offset,
		  ((img[5]*img[3])+img[1])*tile_size,
		  ((img[6]*img[4])+img[2])*tile_size);
      }

      img = (int*)space->data[SPACE_DATA_MIDDLE_IMAGE];
      if(img[0] != -1) {
	//fprintf(stdout,"%s: middle\n",__FUNCTION__);
	draw_tile(images[img[0]],x_offset,y_offset,
		  ((img[5]*img[3])+img[1])*tile_size,
		  ((img[6]*img[4])+img[2])*tile_size);
      }

      /* draw Mobs */
      for(n=0;(mob=((Mob**)space->data[SPACE_DATA_MOBS])[n])!=NULL;n++) {
	//fprintf(stdout,"%s: mob\n",__FUNCTION__);
	img = (int*)mob->data[MOB_DATA_IMAGE];
	draw_tile(images[img[0]],x_offset,y_offset,
		  (img[1]*img[3]*tile_size) + ((x - mob->x) * tile_size),
		  (img[6]*img[5]*img[4]*tile_size) +
		  (img[2]*img[4]*tile_size) + ((y - mob->y) * tile_size));
      }

      img = (int*)space->data[SPACE_DATA_TOP_IMAGE];
      if(img[0] != -1) {
	//fprintf(stdout,"%s: top\n",__FUNCTION__);
	draw_tile(images[img[0]],x_offset,y_offset,
		  ((img[5]*img[3])+img[1])*tile_size,
		  ((img[6]*img[4])+img[2])*tile_size);
      }
    }
    { /* do this once per space... remove it from the update_coords list */
      int *tmp = (int*)update_coords->data;
      update_coords = g_list_remove(update_coords,tmp);
      free(tmp);
    }
  }
  refresh_screen();   /* after all spaces have been drawn, refresh */
  clear_update();

}



/*
 * UTILITY FUNCTIONS USED FOR DRAWING
 */

/* returns boolean, is a coordinate inside a rectangle or not? */
int inside_rect(int x1, int y1, int w, int h, int x2, int y2) {
  return ((x2 >= x1) && (x2 < (x1 + w)) && (y2 >= y1) && (y2 < (y1 + h)));
}

/* use this for scrolling...
 * This is not done yet. I still have to write a hack to make scrolling
 * more intelligent than calling set_viewport() and then add_update_coord()
 * for every space in the viewport...
 */
void set_viewport(int x,int y,int w,int h) {
  viewport[0] = x; viewport[1] = y; viewport[2] = w; viewport[3] = h;
}

void add_update_coord(int x,int y) {
  int *coord = (int*)malloc(sizeof(int)*2);
  coord[0] = x; coord[1] = y;
  update_coords = g_list_prepend(update_coords,coord);
}

void add_update_rect(int x,int y,int w,int h) {
  int i,j;
  for(j=0;j<h;j++)
  for(i=0;i<w;i++) {
    add_update_coord(x+i,y+j);
  }
}

void add_update(int x, int y, int w, int h) {
  SDL_Rect *rects; 

  if((x >= screen_w) || (y >= screen_h)) return;

  rects = (SDL_Rect*)realloc(update,sizeof(SDL_Rect)*(update_size+1));
  update = rects;

  /* handle the clipping that SDL_UpdateRects should do, but doesn't... */
  if(x < 0) rects[update_size].x = 0; 
  else rects[update_size].x = x;
  if(y < 0) rects[update_size].y = 0;
  else rects[update_size].y = y;
  if((x + w) >= screen_w) rects[update_size].w = (screen_w - x) - 1;
  else rects[update_size].w = w;
  if((y + h) >= screen_h) rects[update_size].h = (screen_h - y) - 1;
  else rects[update_size].h = h;

  update_size++;
}

void clear_update() {
  if(update == NULL) return;
  free(update);
  update = NULL;
  update_size = 0;
}

void refresh_screen() {

  /* FIXME: SDL people! SDL_UpdateRects does NOT handle clipping
   * even though it sais so in the documentation.
   */
  SDL_UpdateRects(screen,update_size,update);
  
  
}

void space_frame_set(Space *space,SpaceData image,
		     int x1,int y1,int x2,int y2,int x3,int y3) {
  if((image != SPACE_DATA_BOTTOM_IMAGE) &&
     (image != SPACE_DATA_MIDDLE_IMAGE) &&
     (image != SPACE_DATA_TOP_IMAGE)) {
    fprintf(stderr,"%s: wrong image frame specified %d\n",__FUNCTION__,image);
    return;
  } /* if you know a prettier way, let me know. */
  ((int*)space->data[image])[1] = x1;
  ((int*)space->data[image])[2] = y1;
  ((int*)space->data[image])[3] = x2;
  ((int*)space->data[image])[4] = y2;
  ((int*)space->data[image])[5] = x3;
  ((int*)space->data[image])[6] = y3;
}

void mob_frame_set(Mob *mob,int x1,int y1,int x2,int y2,int x3,int y3) {
  /* if you know a prettier way, let me know. */
  ((int*)mob->data[MOB_DATA_IMAGE])[1] = x1;
  ((int*)mob->data[MOB_DATA_IMAGE])[2] = y1;
  ((int*)mob->data[MOB_DATA_IMAGE])[3] = x2;
  ((int*)mob->data[MOB_DATA_IMAGE])[4] = y2;
  ((int*)mob->data[MOB_DATA_IMAGE])[5] = x3;
  ((int*)mob->data[MOB_DATA_IMAGE])[6] = y3;
}


void map_2_screen_coords(SDL_Rect *rect,int x, int y,int w, int h) {

  rect->x = (x - viewport[0]) * tile_size;
  rect->y = (y - viewport[1]) * tile_size;
  rect->w = w * tile_size;
  rect->h = h * tile_size;

}
