
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include "image.h"

#define RESIZE_FACTOR (2)

GHashTable *image_names = NULL;

SDL_Surface **images = NULL;
int images_size = 0;
int images_used = 0;

void resize_images(int size) {

  SDL_Surface **ni = NULL;
  
  if((ni=(SDL_Surface**)realloc(images,sizeof(SDL_Surface*)*size)) == NULL) {
    fprintf(stderr,"%s: couldn't allocate new images array\n",__FUNCTION__);
    exit(1);
  } else images = ni;

  images_size = size;
  memset(images+images_used,0,(images_size-images_used)*sizeof(SDL_Surface*)); 
  
}

int load_image(char *name) {

  gpointer n;

  if(image_names == NULL)
    image_names = g_hash_table_new(g_str_hash,g_str_equal);
  else if((n = g_hash_table_lookup(image_names,name)) != NULL)
    return GPOINTER_TO_INT(n-1); /* values are 1, 2, ..., NULL = error */

  if(images_size == images_used) {
    if(images_size == 0) images_size = 1;
    resize_images(images_size *= RESIZE_FACTOR); 
  }

  images[images_used] = IMG_Load(name);
  if(images[images_used] == NULL) {
    fprintf(stderr,"Couldn't load %s: %s\n",name,SDL_GetError());
    return -1;
  }

  /* +1 because g_hash_table_insert() returns 0 on failure,
   * while that could be a valid array index in images[]
   */
  g_hash_table_insert(image_names,name,GINT_TO_POINTER(images_used+1));

  return images_used++;

}

