
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include "mob.h"

Mob *mob_new(char *name,int x,int y) {

  Mob *mob = (Mob*)malloc(sizeof(Mob));
  mob->x = x;
  mob->y = y;
  
  /* MobFlags is an enum. of bit offsets, devide it by 8 and you got bytes */
  mob->flags = (Uint8*)malloc((MOB_FLAGS_N/8)+1);
  memset(mob->flags,0,(MOB_FLAGS_N/8)+1);
  
  mob->data = (void**)malloc(sizeof(void*)*MOB_DATA_N);
  memset(mob->data,0,sizeof(void*)*MOB_DATA_N);
  
  if(name != NULL) /* I use that great GLIB utility function g_strdup() */
    mob->data[MOB_DATA_NAME] = g_strdup(name);

  { /* create a brain and seven numbers to hold image information 
       sizeof(int) = 4, which means 4 * 32 bits of information... */
    int i;
    mob->data[MOB_DATA_BRAIN] = (void*)malloc(sizeof(int)*4);
    for(i=0;i<4;i++) ((int*)mob->data[MOB_DATA_BRAIN])[i] = -1;

    mob->data[MOB_DATA_IMAGE] = (void*)malloc(sizeof(int)*7);
    for(i=0;i<7;i++) ((int*)mob->data[MOB_DATA_IMAGE])[i] = -1;
  }
  /* add the monster to the turn list, so that it gets a chance to act. */
  mobs = g_list_prepend(mobs,mob);

  return mob;

}

void mob_destroy(Mob **mob) {

  if((mob == NULL) || (*mob == NULL)) return;

  /* first make sure this mob won't run around like a headless chicken */
  mobs = g_list_remove(mobs,*mob); 

  if((*mob)->data[MOB_DATA_NAME] != NULL) {
    if((*mob)->data[MOB_DATA_NAME] != NULL)
      free((*mob)->data[MOB_DATA_NAME]);
    if((*mob)->data[MOB_DATA_IMAGE] != NULL)
      free((*mob)->data[MOB_DATA_IMAGE]);
    if((*mob)->data[MOB_DATA_BRAIN] != NULL)
      free((*mob)->data[MOB_DATA_BRAIN]);

    free((*mob)->data);
  }
  if((*mob)->flags != NULL) free((*mob)->flags);
  free(*mob);
  *mob = NULL;

}

int space_mob_accessible(Mob *mob, Space *space) {
  if(space!=NULL && !GET_FLAG(*space->flags,SPACE_FLAGS_SOLID) &&
     (((Mob**)space->data[SPACE_DATA_MOBS])[0]==NULL ||
      ((Mob**)space->data[SPACE_DATA_MOBS])[0]==mob)) return 1;
  return 0;
}

Mob* mob_move(Mob *mob,Map *map,int x1, int y1,int x2, int y2) {

  int i,j, *img = (int*)mob->data[MOB_DATA_IMAGE];

  /* img[3] and img[4] specify the size of the Mob in tile_size's
   * When you move a Mob you want to move it entirely, so...
   */
  for(j=0;j<img[4];j++)
    for(i=0;i<img[3];i++)
      if(!space_mob_accessible(mob,(Space*)map_get_space(map,x2+i,y2+j))) 
	return NULL;

  for(j=0;j<img[4];j++) /* it's a shame that we need to do this twice. */
    for(i=0;i<img[3];i++) {
      space_mob_move(mob,
		     (Space*)map_get_space(map,x1+i,y1+j),  /* from */
		     (Space*)map_get_space(map,x2+i,y2+j)); /* to */
      add_update_coord(x1+i,y1+j);
      add_update_coord(x2+i,y2+j);
    }
  mob->x = x2;
  mob->y = y2;
  return mob;

}

Mob* space_mob_move(Mob *mob,Space *from,Space *to) {

  int i,found = 0;
  Mob *tmp = NULL;

  /* if from == NULL then the Mob enters the Space from out of nowhere *grin*
   * and if to == NULL then it leaves the map...
   */
  if(from != NULL) {
    Mob **mobs =  (Mob**)from->data[SPACE_DATA_MOBS] ;
    for(i=0;mobs[i]!=NULL;i++) { /* search for the Mob in the list of mobs */
      if(mobs[i] == mob) {
	tmp = mobs[i]; 
	found = 1;
      }
      if(found)	mobs[i] = mobs[i+1]; /* shift the rest of the Mobs */
    }
    if(found) { /* shrink from->mobs */
      Mob **nm = (Mob**)realloc(mobs,sizeof(Mob*)*i);
      mobs = nm;      
      mobs[i-1] = NULL;
    }
  } else { /* if from == NULL, introduce mob */
    tmp = mob;
  }

  if(to != NULL) {
    Mob **mobs = (Mob**)to->data[SPACE_DATA_MOBS], **nm = NULL;
    for(i=0;mobs[i]!=NULL;i++); /* count how many Mobs there are in mobs */
    nm = (Mob**)realloc(mobs,sizeof(Mob*)*(i+2)); /* grow mobs by 1 */
    mobs = nm;      
    mobs[i] = tmp; /* add our mob to the list of mobs */
    mobs[i+1] = NULL; /* terminate mobs with a NULL pointer */
  }

  /* DEBUG STUFF... */
  //if(from != NULL)
  //  for(i=0;((Mob**)from->data[SPACE_DATA_MOBS])[i]!=NULL;i++)
  //    fprintf(stdout,"%s: %s from mobs(%d,%d) (%X)\n",__FUNCTION__,
  //	      tmp->data[MOB_DATA_NAME],
  //      from->x,from->y,((Mob**)to->data[SPACE_DATA_MOBS])[i]);
  //if(to != NULL)
  //  for(i=0;((Mob**)to->data[SPACE_DATA_MOBS])[i]!=NULL;i++)
  //    fprintf(stdout,"%s: %s to mobs(%d,%d) (%X)\n",__FUNCTION__,
  //        tmp->data[MOB_DATA_NAME],
  //      to->x,to->y,((Mob**)to->data[SPACE_DATA_MOBS])[i]);
  
  return tmp;

}

/* there's no need to implement seperate add and remove functions,
 * since space_mob_move() will do the job just fine...
 */
void space_mob_add(Mob *mob,Space *space) {
  (void*)space_mob_move(mob,NULL,space);
}
void space_mob_remove(Mob *mob,Space *space) {
  (void*)space_mob_move(mob,space,NULL);
}

void mob_destroy_gfunc(gpointer data,gpointer user_data) {

  Mob *mob = (Mob*)data;

  if(mob == NULL) return;
  mob_destroy(&mob);

}
