
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#include "turn.h" /* this includes everything we need,
		   * including all the global variables, like "map"
		   */

#include <sys/time.h> /* DEBUG */
#include <unistd.h>

Uint32 turn_delay = 50; /* msecs */
Uint32 current_time = 0;
Uint8 *game_flags = NULL;
GList *mobs = NULL;


void mob_behavior_gfunc(gpointer data,gpointer user_data) {

  Mob *mob = (Mob*)data;

  if((mob == NULL) || (mob->data == NULL) ||
     ((int)mob->data[MOB_DATA_BEHAVIOR] == -1)) return;
  (*behavior_funcs[(int)mob->data[MOB_DATA_BEHAVIOR]])(map,mob);

}

void game_loop(void) {
  
  if(game_flags == NULL) { /* new game */
    game_flags = (Uint8*)malloc((GAME_FLAGS_N/8)+1);
    memset(game_flags,0,(GAME_FLAGS_N/8)+1);
    SET_FLAG(*game_flags,GAME_FLAGS_RUNNING);
  }

  while(GET_FLAG(*game_flags,GAME_FLAGS_RUNNING)) {
    g_list_foreach(mobs,mob_behavior_gfunc,NULL); /* give every Mob a turn */
    draw_map(map); /* update the map */
    parse_events(turn_delay); /* spend the rest of the time on input */
  }

  /* end of game */
  if(game_flags != NULL) free(game_flags);
  g_list_foreach(mobs,mob_destroy_gfunc,NULL); /* destroy all Mobs */

  if(map != NULL) map_destroy(&map); /* this should destroy everything else */

}

