
/* Copyright (C) 2001 Willem Robert van Hage
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can get it from http://www.fsf.org
 * or write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef QUEST_TYPES_H
#define QUEST_TYPES_H

#include "SDL.h"
#include "SDL_image.h"
#include <glib.h>
#include <gmodule.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aux.h"

/* The enumerations and types describe how the game will be.
 * Many of them aren't used yet... but feel free to add functionality.
 * I try not to write anything that depends on hard wired offsets,
 * so that you can add anything anywhere in the structs or enums.
 * 
 * Willem
 */

typedef enum GameFlags {

  /* bit offsets */
  GAME_FLAGS_RUNNING,
  GAME_FLAGS_PAUSED,
  GAME_FLAGS_N

} GameFlags;

/* I use the ..._N's to determine the number of fields in the enum. */
typedef enum SpaceFlags {

  /* bit offsets */
  SPACE_FLAGS_SOLID,
  SPACE_FLAGS_SIT, /* trigger sit image for mobs */
  SPACE_FLAGS_SLEEP, /* trigger sleep image for mobs */
  SPACE_FLAGS_N

} SpaceFlags;


typedef enum SpaceData {

  /* byte offsets */
  SPACE_NAME,              /* grass, or a big tree, or something... */
  SPACE_DATA_BOTTOM_IMAGE, /* grass, dirt, water */
  SPACE_DATA_MIDDLE_IMAGE, /* tree trunks, boulders, fences */
  SPACE_DATA_TOP_IMAGE,    /* branches, a roof, archways */
  SPACE_DATA_TELEPORTER,   /* this space connects to another place */
  SPACE_DATA_MOBS,         /* movable objects, a NULL terminated list */
  SPACE_DATA_N

} SpaceData;

typedef struct SpaceTeleporter {

  char *host;
  int port;

  /* the x and y of the destination Space */
  int abs_x; 
  int abs_y;

  /* where the destination Space should be drawn,
     relative to this Space */
  int rel_x;
  int rel_y;

} SpaceTeleporter;


typedef struct Space {

  int x;
  int y;

  Uint8 *flags; /* a bunch of bytes */
  void **data;  /* the rest of the data we need, indexed by enum SpaceData */

} Space;


typedef struct Map {

  GHashTable *spaces;

} Map;

typedef enum MobFlags {

  /* bit offsets */
  MOB_FLAGS_PLAYER,
  MOB_FLAGS_DEAD,
  MOB_FLAGS_POISON,
  MOB_FLAGS_SLEEP,
  MOB_FLAGS_HOSTILE,
  MOB_FLAGS_N

} MobFlags;

typedef enum MobData {

  /* byte offsets */
  MOB_DATA_NAME,     /* a string */
  MOB_DATA_IMAGE,    /* an int key to the SDL_Surface */
  MOB_DATA_BEHAVIOR, /* pointer to a behavior function */
  MOB_DATA_BRAIN,    /* a small byte array of data for behavior */
  MOB_DATA_CONTENTS, /* a list of pointers ending in NULL */
  MOB_DATA_N

} MobData;

typedef struct Mob {

  int x;
  int y;
  /* you can deduce the width and height of a Mob
   * from its image index [3] and [4].
   * so there's no need for two extra ints.
   */

  Uint8 *flags;
  void **data;

} Mob;


#endif /* QUEST_TYPES_H */
